@@ -13,38 +13,21 @@
 #    under the License.
 
 import json
+import types
 
+from django.contrib.staticfiles.templatetags.staticfiles import static
+from django.core.urlresolvers import reverse
 from django.template import loader
-from django.templatetags.static import static
-from django.urls import reverse
 
 from muranodashboard.api import packages as pkg_cli
-from muranodashboard.environments import consts
 
 
-def get_app_image(request, app_fqdn, status=None):
-    if '@' in app_fqdn:
-        class_fqn, package_fqn = app_fqdn.split('@')
-        if '/' in class_fqn:
-            class_fqn, version = class_fqn.split('/')
-        else:
-            version = None
-    else:
-        package_fqn = app_fqdn
-        version = None
-    package = pkg_cli.app_by_fqn(request, package_fqn, version=version)
-    if status in [
-       consts.STATUS_ID_DEPLOY_FAILURE,
-       consts.STATUS_ID_DELETE_FAILURE,
-       ]:
-        url = static('dashboard/img/stack-red.svg')
-    elif status == consts.STATUS_ID_READY:
-        url = static('dashboard/img/stack-green.svg')
-    else:
-        url = static('dashboard/img/stack-gray.svg')
+def get_app_image(request, app_fqdn):
+    package = pkg_cli.app_by_fqn(request, app_fqdn)
+    url = static('dashboard/img/stack-green.svg')
     if package:
         app_id = package.id
-        url = reverse("horizon:app-catalog:catalog:images", args=(app_id,))
+        url = reverse("horizon:murano:catalog:images", args=(app_id,))
     return url
 
 
@@ -56,16 +39,14 @@ def _get_environment_status_message(entity):
 
     in_progress = True
     status_message = ''
-    if status in (consts.STATUS_ID_PENDING, consts.STATUS_ID_READY):
+    if status in ('pending', 'ready'):
         in_progress = False
-    if status == consts.STATUS_ID_PENDING:
+    if status == 'pending':
         status_message = 'Waiting for deployment'
-    elif status == consts.STATUS_ID_READY:
+    elif status == 'ready':
         status_message = 'Deployed'
-    elif status == consts.STATUS_ID_DEPLOYING:
+    elif status == 'deploying':
         status_message = 'Deployment is in progress'
-    elif status == consts.STATUS_ID_DEPLOY_FAILURE:
-        status_message = 'Deployment failed'
     return in_progress, status_message
 
 
@@ -85,10 +66,7 @@ def _truncate_type(type_str, num_of_chars):
 
 
 def _application_info(application, app_image, status):
-    name = application['?'].get('name')
-    if not name:
-        name = application.get('name')
-    context = {'name': name,
+    context = {'name': application['name'],
                'type': _truncate_type(application['?']['type'], 45),
                'status': status,
                'app_image': app_image}
@@ -148,14 +126,6 @@ def _create_ext_network_node(name):
     return node
 
 
-def _convert_lists(node_data):
-    for key, value in node_data.items():
-        if isinstance(value, list) and all(
-                map(lambda s: not isinstance(s, (dict, list)), value)):
-            new_value = ', '.join(str(v) for v in value)
-            node_data[key] = new_value
-
-
 def _split_seq_by_predicate(seq, predicate):
     holds, not_holds = [], []
     for elt in seq:
@@ -168,11 +138,20 @@ def _split_seq_by_predicate(seq, predicate):
 
 def _is_atomic(elt):
     key, value = elt
-    return not isinstance(value, (dict, list))
+    return not isinstance(value, (types.DictType, types.ListType))
+
+
+def check_service_references(node, node_data, srv_list):
+    node_id = node_data['?']['id']
+    for service in srv_list:
+        for k, v in service.iteritems():
+            if v == node_id:
+                node['required_by'].append(service['?']['id'])
+                node['link_type'] = 'relation'
 
 
 def render_d3_data(request, environment):
-    if not (environment and environment.services):
+    if not environment:
         return None
 
     ext_net_name = None
@@ -193,87 +172,10 @@ def render_d3_data(request, environment):
     unit_image_active = static('dashboard/img/server-green.svg')
     unit_image_non_active = static('dashboard/img/server-gray.svg')
 
-    node_refs = {}
-
-    def get_image(fqdn, node_data):
-        if fqdn.startswith('io.murano.resources'):
-            if len(node_data.get('ipAddresses', [])) > 0:
-                image = unit_image_active
-            else:
-                image = unit_image_non_active
-        else:
-            image = get_app_image(request, fqdn)
-        return image
-
-    def rec(node_data, node_key, parent_node=None):
-        if not isinstance(node_data, dict):
-            return
-        _convert_lists(node_data)
-        node_type = node_data.get('?', {}).get('type')
-        node_id = node_data.get('?', {}).get('id')
-        atomics, containers = _split_seq_by_predicate(
-            node_data.items(), _is_atomic)
-        if node_type and node_data is not parent_node:
-            node = _create_empty_node()
-            node_refs[node_id] = node
-            atomics.extend([('id', node_data['?']['id']),
-                            ('type', node_type),
-                            ('name', node_data.get('name', node_key))])
-
-            image = get_image(node_type, node_data)
-            node.update({
-                'id': node_id,
-                'info_box': _unit_info(atomics, image),
-                'image': image,
-                'link_type': 'unit',
-                'in_progress': in_progress})
-            d3_data['nodes'].append(node)
-
-        for key, value in containers:
-            if key == '?':
-                continue
-            if isinstance(value, dict):
-                rec(value, key, node_data)
-            elif isinstance(value, list):
-                for index, val in enumerate(value):
-                    rec(val, '{0}[{1}]'.format(key, index), node_data)
-
-    def build_links_rec(node_data, parent_node=None):
-        if not isinstance(node_data, dict):
-            return
-        node_id = node_data.get('?', {}).get('id')
-        if not node_id:
-            return
-        node = node_refs[node_id]
-
-        atomics, containers = _split_seq_by_predicate(
-            node_data.items(), _is_atomic)
-
-        # the actual second pass of node linking
-        if parent_node is not None:
-            node['required_by'].append(parent_node['?']['id'])
-            node['link_type'] = 'aggregation'
-
-        for key, value in atomics:
-            if value in node_refs:
-                remote_node = node_refs[value]
-                if node_id not in remote_node['required_by']:
-                    remote_node['required_by'].append(node_id)
-                    remote_node['link_type'] = 'reference'
-
-        for key, value in containers:
-            if key == '?':
-                continue
-            if isinstance(value, dict):
-                build_links_rec(value, node_data)
-            elif isinstance(value, list):
-                for val in value:
-                    build_links_rec(val, node_data)
-
     for service in environment.services:
         in_progress, status_message = _get_environment_status_message(service)
         required_by = None
-        if 'instance' in service and service['instance'] is not None:
+        if 'instance' in service:
             if service['instance'].get('assignFloatingIp', False):
                 if ext_net_name:
                     required_by = ext_net_name
@@ -284,15 +186,12 @@ def render_d3_data(request, environment):
                     required_by = ext_net_name
 
         service_node = _create_empty_node()
-        service_image = get_app_image(request, service['?']['type'],
-                                      service['?']['status'])
-        node_id = service['?']['id']
-        node_refs[node_id] = service_node
+        service_image = get_app_image(request, service['?']['type'])
         service_node.update({
             'name': service.get('name', ''),
             'status': status_message,
             'image': service_image,
-            'id': node_id,
+            'id': service['?']['id'],
             'link_type': 'relation',
             'in_progress': in_progress,
             'info_box': _application_info(
@@ -300,10 +199,43 @@ def render_d3_data(request, environment):
         })
         if required_by:
             service_node['required_by'].append(required_by)
+        check_service_references(service_node, service, environment.services)
         d3_data['nodes'].append(service_node)
+
+        def rec(node_data, node_key, parent_node=None):
+            if not isinstance(node_data, types.DictType):
+                return
+            node_type = node_data.get('?', {}).get('type')
+            atomics, containers = _split_seq_by_predicate(
+                node_data.iteritems(), _is_atomic)
+            if node_type and node_data is not parent_node:
+                node = _create_empty_node()
+                atomics.extend([('id', node_data['?']['id']),
+                                ('type', node_type),
+                                ('name', node_data.get('name', node_key))])
+                if parent_node is not None:
+                    node['required_by'].append(parent_node['?']['id'])
+                if len(node_data.get('ipAddresses', [])) > 0:
+                    image = unit_image_active
+                else:
+                    image = unit_image_non_active
+                node.update({
+                    'id': node_data['?']['id'],
+                    'info_box': _unit_info(atomics, image),
+                    'image': image,
+                    'link_type': 'unit',
+                    'in_progress': in_progress})
+                d3_data['nodes'].append(node)
+
+            for key, value in containers:
+                if key == '?':
+                    continue
+                if isinstance(value, types.DictType):
+                    rec(value, key, node_data)
+                elif isinstance(value, types.ListType):
+                    for index, val in enumerate(value):
+                        rec(val, '{0}[{1}]'.format(key, index), node_data)
+
         rec(service, None, service)
 
-    for service in environment.services:
-        build_links_rec(service)
-
     return json.dumps(d3_data)
